==> lkrg-0.5.tar.gz: lkrg-0.5/src/modules/exploit_detection/syscalls/__x32/p_x32_sys_ptrace/p_x32_sys_ptrace.c <==
/*
 * pi3's Linux kernel Runtime Guard
 *
 * Component:
 *  - Intercept X32 ptrace syscall
 *
 * Notes:
 *  - Enforce Exploit Detection validation
 *
 * Caveats:
 *  - None
 *
 * Timeline:
 *  - Created: 13.VIII.2018
 *
 * Author:
 *  - Adam 'pi3' Zabrocki (http://pi3.com.pl)
 *
 */

#ifdef CONFIG_X86_X32

#include "../../../../../p_lkrg_main.h"

#ifdef P_SYSCALL_LAYOUT_4_17


char p_x32_sys_ptrace_kretprobe_state = 0x0;

static struct kretprobe p_x32_sys_ptrace_kretprobe = {
    .kp.symbol_name = P_GET_X32_SYSCALL_NAME(ptrace),
    .handler = p_x32_sys_ptrace_ret,
    .entry_handler = p_x32_sys_ptrace_entry,
    .data_size = sizeof(struct p_x32_sys_ptrace_data),
    /* Probe up to 40 instances concurrently. */
    .maxactive = 40,
};

/*
 * x86-64 syscall ABI:
 *  *rax - syscall_number
 *    rdi - 1st argument
 *    rsi - 2nd argument
 *    rdx - 3rd argument
 *    rcx - 4th argument
 *
 *    r8  - 5th one
 *    r9  - 6th one
 */

int p_x32_sys_ptrace_entry(struct kretprobe_instance *p_ri, struct pt_regs *p_regs) {

   struct p_ed_process *p_tmp;

// STRONG_DEBUG
   p_debug_kprobe_log(
          "Entering function <p_x32_sys_ptrace_entry>\n");

   spin_lock(&p_rb_ed_pids_lock);
   if ( (p_tmp = p_rb_find_ed_pid(&p_global_ed_pids_root, task_pid_nr(current))) != NULL) {
      // This process is on the ED list - validate 'off' flag
      p_ed_is_off_off_wrap(p_tmp);
   }
   spin_unlock(&p_rb_ed_pids_lock);

   p_ed_enforce_validation();

// STRONG_DEBUG
   p_debug_kprobe_log(
          "Leaving function <p_x32_sys_ptrace_entry>\n");

   return 0x0;
}


int p_x32_sys_ptrace_ret(struct kretprobe_instance *p_ri, struct pt_regs *p_regs) {

// STRONG_DEBUG
   p_debug_kprobe_log(
          "Entering function <p_x32_sys_ptrace_ret>\n");

//   p_ed_enforce_validation();

// STRONG_DEBUG
   p_debug_kprobe_log(
          "Leaving function <p_x32_sys_ptrace_ret>\n");

   return 0x0;
}


int p_install_x32_sys_ptrace_hook(void) {

   int p_ret;

// STRONG_DEBUG
   p_debug_log(P_LKRG_STRONG_DBG,
          "Entering function <p_install_x32_sys_ptrace_hook>\n");

   if ( (p_ret = register_kretprobe(&p_x32_sys_ptrace_kretprobe)) < 0) {
      p_print_log(P_LKRG_ERR, "[kretprobe] register_kretprobe() for <%s> failed! [err=%d]\n",
                  p_x32_sys_ptrace_kretprobe.kp.symbol_name,
                  p_ret);
      goto p_install_x32_sys_ptrace_hook_out;
   }
   p_print_log(P_LKRG_INFO, "Planted [kretprobe] <%s> at: %p\n",
               p_x32_sys_ptrace_kretprobe.kp.symbol_name,
               p_x32_sys_ptrace_kretprobe.kp.addr);
   p_x32_sys_ptrace_kretprobe_state = 0x1;

//   p_ret = 0x0; <- should be 0x0 anyway...

p_install_x32_sys_ptrace_hook_out:

// STRONG_DEBUG
   p_debug_log(P_LKRG_STRONG_DBG,
          "Leaving function <p_install_x32_sys_ptrace_hook> (p_ret => %d)\n",p_ret);

   return p_ret;
}


void p_uninstall_x32_sys_ptrace_hook(void) {

// STRONG_DEBUG
   p_debug_log(P_LKRG_STRONG_DBG,
          "Entering function <p_uninstall_x32_sys_ptrace_hook>\n");

   if (!p_x32_sys_ptrace_kretprobe_state) {
      p_print_log(P_LKRG_INFO, "[kretprobe] <%s> at 0x%p is NOT installed\n",
                  p_x32_sys_ptrace_kretprobe.kp.symbol_name,
                  p_x32_sys_ptrace_kretprobe.kp.addr);
   } else {
      unregister_kretprobe(&p_x32_sys_ptrace_kretprobe);
      p_print_log(P_LKRG_INFO, "Removing [kretprobe] <%s> at 0x%p nmissed[%d]\n",
                  p_x32_sys_ptrace_kretprobe.kp.symbol_name,
                  p_x32_sys_ptrace_kretprobe.kp.addr,
                  p_x32_sys_ptrace_kretprobe.nmissed);
      p_x32_sys_ptrace_kretprobe_state = 0x0;
   }

// STRONG_DEBUG
   p_debug_log(P_LKRG_STRONG_DBG,
          "Leaving function <p_uninstall_x32_sys_ptrace_hook>\n");
}

#endif

#endif

==> lkrg-0.8.tar.gz: lkrg-0.8/src/modules/exploit_detection/syscalls/__x32/p_x32_sys_ptrace/p_x32_sys_ptrace.c <==
/*
 * pi3's Linux kernel Runtime Guard
 *
 * Component:
 *  - Intercept X32 ptrace syscall
 *
 * Notes:
 *  - Enforce Exploit Detection validation
 *
 * Caveats:
 *  - None
 *
 * Timeline:
 *  - Created: 13.VIII.2018
 *
 * Author:
 *  - Adam 'pi3' Zabrocki (http://pi3.com.pl)
 *
 */

#ifdef CONFIG_X86_X32

#include "../../../../../p_lkrg_main.h"

#ifdef P_SYSCALL_LAYOUT_4_17


char p_x32_sys_ptrace_kretprobe_state = 0x0;

static struct kretprobe p_x32_sys_ptrace_kretprobe = {
    .kp.symbol_name = P_GET_X32_SYSCALL_NAME(ptrace),
    .handler = p_x32_sys_ptrace_ret,
    .entry_handler = p_x32_sys_ptrace_entry,
    .data_size = sizeof(struct p_x32_sys_ptrace_data),
    /* Probe up to 40 instances concurrently. */
    .maxactive = 40,
};

/*
 * x86-64 syscall ABI:
 *  *rax - syscall_number
 *    rdi - 1st argument
 *    rsi - 2nd argument
 *    rdx - 3rd argument
 *    rcx - 4th argument
 *
 *    r8  - 5th one
 *    r9  - 6th one
 */

int p_x32_sys_ptrace_entry(struct kretprobe_instance *p_ri, struct pt_regs *p_regs) {

   struct p_ed_process *p_tmp;
   unsigned long p_flags;

// STRONG_DEBUG
   p_debug_kprobe_log(
          "Entering function <p_x32_sys_ptrace_entry>\n");

   p_tasks_read_lock(&p_flags);
   if ( (p_tmp = p_find_ed_by_pid(task_pid_nr(current))) != NULL) {
      // This process is on the ED list - validate 'off' flag
      p_ed_is_off_off_wrap(p_tmp);
   }
   p_tasks_read_unlock(&p_flags);

   p_ed_enforce_validation();

// STRONG_DEBUG
   p_debug_kprobe_log(
          "Leaving function <p_x32_sys_ptrace_entry>\n");

   return 0x0;
}


int p_x32_sys_ptrace_ret(struct kretprobe_instance *p_ri, struct pt_regs *p_regs) {

// STRONG_DEBUG
   p_debug_kprobe_log(
          "Entering function <p_x32_sys_ptrace_ret>\n");

//   p_ed_enforce_validation();

// STRONG_DEBUG
   p_debug_kprobe_log(
          "Leaving function <p_x32_sys_ptrace_ret>\n");

   return 0x0;
}


GENERATE_INSTALL_FUNC(x32_sys_ptrace)

#endif

#endif
